@@ -1,6 +1,6 @@
 /* Handle JIT code generation in the inferior for GDB, the GNU Debugger.
 
-   Copyright (C) 2009-2022 Free Software Foundation, Inc.
+   Copyright (C) 2009-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,9 +17,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 
 #include "jit.h"
+#include "extract-store-integer.h"
 #include "jit-reader.h"
 #include "block.h"
 #include "breakpoint.h"
@@ -27,7 +27,8 @@
 #include "dictionary.h"
 #include "filenames.h"
 #include "frame-unwind.h"
-#include "gdbcmd.h"
+#include "cli/cli-cmds.h"
+#include "cli/cli-style.h"
 #include "gdbcore.h"
 #include "inferior.h"
 #include "observable.h"
@@ -39,7 +40,6 @@
 #include "gdbsupport/gdb-dlfcn.h"
 #include <sys/stat.h>
 #include "gdb_bfd.h"
-#include "readline/tilde.h"
 #include "completer.h"
 #include <forward_list>
 
@@ -76,12 +76,12 @@ maint_info_jit_cmd (const char *args, int from_tty)
   inferior *inf = current_inferior ();
   bool printed_header = false;
 
-  gdb::optional<ui_out_emit_table> table_emitter;
+  std::optional<ui_out_emit_table> table_emitter;
 
   /* Print a line for each JIT-ed objfile.  */
-  for (objfile *obj : inf->pspace->objfiles ())
+  for (objfile &obj : inf->pspace->objfiles ())
     {
-      if (obj->jited_data == nullptr)
+      if (obj.jited_data == nullptr)
 	continue;
 
       if (!printed_header)
@@ -90,7 +90,7 @@ maint_info_jit_cmd (const char *args, int from_tty)
 
 	  /* The +2 allows for the leading '0x', then one character for
 	     every 4-bits.  */
-	  int addr_width = 2 + (gdbarch_ptr_bit (obj->arch ()) / 4);
+	  int addr_width = 2 + (gdbarch_ptr_bit (obj.arch ()) / 4);
 
 	  /* The std::max here selects between the width of an address (as
 	     a string) and the width of the column header string.  */
@@ -108,12 +108,12 @@ maint_info_jit_cmd (const char *args, int from_tty)
 
       ui_out_emit_tuple tuple_emitter (current_uiout, "jit-objfile");
 
-      current_uiout->field_core_addr ("jit_code_entry-address", obj->arch (),
-				      obj->jited_data->addr);
-      current_uiout->field_core_addr ("symfile-address", obj->arch (),
-				      obj->jited_data->symfile_addr);
+      current_uiout->field_core_addr ("jit_code_entry-address", obj.arch (),
+				      obj.jited_data->addr);
+      current_uiout->field_core_addr ("symfile-address", obj.arch (),
+				      obj.jited_data->symfile_addr);
       current_uiout->field_unsigned ("symfile-size",
-				      obj->jited_data->symfile_size);
+				      obj.jited_data->symfile_size);
       current_uiout->text ("\n");
     }
 }
@@ -178,7 +178,8 @@ jit_reader_load_command (const char *args, int from_tty)
 {
   if (args == NULL)
     error (_("No reader name provided."));
-  gdb::unique_xmalloc_ptr<char> file (tilde_expand (args));
+  gdb::unique_xmalloc_ptr<char> file
+    = gdb_rl_tilde_expand (args);
 
   if (loaded_jit_reader != NULL)
     error (_("JIT reader already loaded.  Run jit-reader-unload first."));
@@ -222,7 +223,7 @@ static jiter_objfile_data *
 get_jiter_objfile_data (objfile *objf)
 {
   if (objf->jiter_data == nullptr)
-    objf->jiter_data.reset (new jiter_objfile_data ());
+    objf->jiter_data = std::make_unique<jiter_objfile_data> ();
 
   return objf->jiter_data.get ();
 }
@@ -236,8 +237,9 @@ add_objfile_entry (struct objfile *objfile, CORE_ADDR entry,
 {
   gdb_assert (objfile->jited_data == nullptr);
 
-  objfile->jited_data.reset (new jited_objfile_data (entry, symfile_addr,
-						     symfile_size));
+  objfile->jited_data = std::make_unique<jited_objfile_data> (entry,
+							      symfile_addr,
+							      symfile_size);
 }
 
 /* Helper function for reading the global JIT descriptor from remote
@@ -491,9 +493,10 @@ jit_symtab_line_mapping_add_impl (struct gdb_symbol_callbacks *cb,
   stab->linetable->nitems = nlines;
   for (i = 0; i < nlines; i++)
     {
-      stab->linetable->item[i].pc = (CORE_ADDR) map[i].pc;
+      stab->linetable->item[i].set_unrelocated_pc
+	(unrelocated_addr (map[i].pc));
       stab->linetable->item[i].line = map[i].line;
-      stab->linetable->item[i].is_stmt = 1;
+      stab->linetable->item[i].is_stmt = true;
     }
 }
 
@@ -514,10 +517,7 @@ jit_symtab_close_impl (struct gdb_symbol_callbacks *cb,
 static void
 finalize_symtab (struct gdb_symtab *stab, struct objfile *objfile)
 {
-  struct compunit_symtab *cust;
-  size_t blockvector_size;
   CORE_ADDR begin, end;
-  struct blockvector *bv;
 
   int actual_nblocks = FIRST_LOCAL_BLOCK + stab->nblocks;
 
@@ -530,9 +530,11 @@ finalize_symtab (struct gdb_symtab *stab, struct objfile *objfile)
       return a.end > b.end;
     });
 
-  cust = allocate_compunit_symtab (objfile, stab->file_name.c_str ());
+  auto cusymtab = std::make_unique<compunit_symtab> (objfile,
+						     stab->file_name.c_str ());
+  compunit_symtab *cust
+    = add_compunit_symtab_to_objfile (std::move (cusymtab));
   symtab *filetab = allocate_symtab (cust, stab->file_name.c_str ());
-  add_compunit_symtab_to_objfile (cust);
 
   /* JIT compilers compile in memory.  */
   cust->set_dirname (nullptr);
@@ -543,23 +545,19 @@ finalize_symtab (struct gdb_symtab *stab, struct objfile *objfile)
       size_t size = ((stab->linetable->nitems - 1)
 		     * sizeof (struct linetable_entry)
 		     + sizeof (struct linetable));
-      filetab->set_linetable ((struct linetable *)
-			      obstack_alloc (&objfile->objfile_obstack, size));
-      memcpy (filetab->linetable (), stab->linetable.get (), size);
+      struct linetable *new_table
+	= (struct linetable *) obstack_alloc (&objfile->objfile_obstack,
+					      size);
+      memcpy (new_table, stab->linetable.get (), size);
+      filetab->set_linetable (new_table);
     }
 
-  blockvector_size = (sizeof (struct blockvector)
-		      + (actual_nblocks - 1) * sizeof (struct block *));
-  bv = (struct blockvector *) obstack_alloc (&objfile->objfile_obstack,
-					     blockvector_size);
-  cust->set_blockvector (bv);
+  auto bv = std::make_unique<blockvector> (actual_nblocks);
 
   /* At the end of this function, (begin, end) will contain the PC range this
      entire blockvector spans.  */
-  bv->set_map (nullptr);
   begin = stab->blocks.front ().begin;
   end = stab->blocks.front ().end;
-  bv->set_num_blocks (actual_nblocks);
 
   /* First run over all the gdb_block objects, creating a real block
      object for each.  Simultaneously, keep setting the real_block
@@ -567,22 +565,19 @@ finalize_symtab (struct gdb_symtab *stab, struct objfile *objfile)
   int block_idx = FIRST_LOCAL_BLOCK;
   for (gdb_block &gdb_block_iter : stab->blocks)
     {
-      struct block *new_block = allocate_block (&objfile->objfile_obstack);
+      struct block *new_block = new (&objfile->objfile_obstack) block;
       struct symbol *block_name = new (&objfile->objfile_obstack) symbol;
-      struct type *block_type = arch_type (objfile->arch (),
-					   TYPE_CODE_VOID,
-					   TARGET_CHAR_BIT,
-					   "void");
+      struct type *block_type = builtin_type (objfile->arch ())->builtin_void;
 
       new_block->set_multidict
-	(mdict_create_linear (&objfile->objfile_obstack, NULL));
+	(mdict_create_linear (&objfile->objfile_obstack, {}));
       /* The address range.  */
       new_block->set_start (gdb_block_iter.begin);
       new_block->set_end (gdb_block_iter.end);
 
       /* The name.  */
-      block_name->set_domain (VAR_DOMAIN);
-      block_name->set_aclass_index (LOC_BLOCK);
+      block_name->set_domain (FUNCTION_DOMAIN);
+      block_name->set_loc_class_index (LOC_BLOCK);
       block_name->set_symtab (filetab);
       block_name->set_type (lookup_function_type (block_type));
       block_name->set_value_block (new_block);
@@ -609,11 +604,13 @@ finalize_symtab (struct gdb_symtab *stab, struct objfile *objfile)
     {
       struct block *new_block;
 
-      new_block = (i == GLOBAL_BLOCK
-		   ? allocate_global_block (&objfile->objfile_obstack)
-		   : allocate_block (&objfile->objfile_obstack));
+      if (i == GLOBAL_BLOCK)
+	new_block = new (&objfile->objfile_obstack) global_block;
+      else
+	new_block = new (&objfile->objfile_obstack) block;
+
       new_block->set_multidict
-	(mdict_create_linear (&objfile->objfile_obstack, NULL));
+	(mdict_create_linear (&objfile->objfile_obstack, {}));
       new_block->set_superblock (block_iter);
       block_iter = new_block;
 
@@ -623,7 +620,7 @@ finalize_symtab (struct gdb_symtab *stab, struct objfile *objfile)
       bv->set_block (i, new_block);
 
       if (i == GLOBAL_BLOCK)
-	set_block_compunit_symtab (new_block, cust);
+	new_block->as_global_block ()->set_compunit (cust);
     }
 
   /* Fill up the superblock fields for the real blocks, using the
@@ -644,6 +641,9 @@ finalize_symtab (struct gdb_symtab *stab, struct objfile *objfile)
 	  gdb_block_iter.real_block->set_superblock (bv->static_block ());
 	}
     }
+
+  /* Move just built blockvector over to CUST.  */
+  cust->set_blockvector (std::move (bv));
 }
 
 /* Called when closing a gdb_objfile.  Converts OBJ to a proper
@@ -659,8 +659,10 @@ jit_object_close_impl (struct gdb_symbol_callbacks *cb,
 		     paddress (priv_data->gdbarch,
 			       priv_data->entry.symfile_addr));
 
-  objfile *objfile = objfile::make (nullptr, objfile_name.c_str (),
-				    OBJF_NOT_FILENAME);
+  objfile *objfile = objfile::make (nullptr, current_program_space,
+				    objfile_name.c_str (), OBJF_NOT_FILENAME);
+  objfile->section_offsets.push_back (0);
+  objfile->sect_index_text = 0;
   objfile->per_bfd->gdbarch = priv_data->gdbarch;
 
   for (gdb_symtab &symtab : obj->symtabs)
@@ -715,7 +717,7 @@ jit_reader_try_read_symtab (gdbarch *gdbarch, jit_code_entry *code_entry,
 			      code_entry->symfile_size))
 	status = 0;
     }
-  catch (const gdb_exception &e)
+  catch (const gdb_exception_error &e)
     {
       status = 0;
     }
@@ -778,8 +780,8 @@ JITed symbol file is not an object file, ignoring it.\n"));
 	     b->printable_name);
 
   /* Read the section address information out of the symbol file.  Since the
-     file is generated by the JIT at runtime, it should all of the absolute
-     addresses that we care about.  */
+     file is generated by the JIT at runtime, it should contain all of the
+     absolute addresses that we care about.  */
   section_addr_info sai;
   for (sec = nbfd->sections; sec != NULL; sec = sec->next)
     if ((bfd_section_flags (sec) & (SEC_ALLOC|SEC_LOAD)) != 0)
@@ -827,10 +829,10 @@ jit_register_code (struct gdbarch *gdbarch,
 static struct objfile *
 jit_find_objf_with_entry_addr (CORE_ADDR entry_addr)
 {
-  for (objfile *objf : current_program_space->objfiles ())
+  for (objfile &objf : current_program_space->objfiles ())
     {
-      if (objf->jited_data != nullptr && objf->jited_data->addr == entry_addr)
-	return objf;
+      if (objf.jited_data != nullptr && objf.jited_data->addr == entry_addr)
+	return &objf;
     }
 
   return NULL;
@@ -845,14 +847,14 @@ jit_breakpoint_deleted (struct breakpoint *b)
   if (b->type != bp_jit_event)
     return;
 
-  for (bp_location *iter : b->locations ())
+  for (bp_location &iter : b->locations ())
     {
-      for (objfile *objf : iter->pspace->objfiles ())
+      for (objfile &objf : iter.pspace->objfiles ())
 	{
-	  jiter_objfile_data *jiter_data = objf->jiter_data.get ();
+	  jiter_objfile_data *jiter_data = objf.jiter_data.get ();
 
 	  if (jiter_data != nullptr
-	      && jiter_data->jit_breakpoint == iter->owner)
+	      && jiter_data->jit_breakpoint == iter.owner)
 	    {
 	      jiter_data->cached_code_address = 0;
 	      jiter_data->jit_breakpoint = nullptr;
@@ -867,43 +869,44 @@ jit_breakpoint_deleted (struct breakpoint *b)
 static void
 jit_breakpoint_re_set_internal (struct gdbarch *gdbarch, program_space *pspace)
 {
-  for (objfile *the_objfile : pspace->objfiles ())
+  for (objfile &the_objfile : pspace->objfiles ())
     {
       /* Skip separate debug objects.  */
-      if (the_objfile->separate_debug_objfile_backlink != nullptr)
+      if (the_objfile.separate_debug_objfile_backlink != nullptr)
 	continue;
 
-      if (the_objfile->skip_jit_symbol_lookup)
+      if (the_objfile.skip_jit_symbol_lookup)
 	continue;
 
       /* Lookup the registration symbol.  If it is missing, then we
 	 assume we are not attached to a JIT.  */
       bound_minimal_symbol reg_symbol
-	= lookup_minimal_symbol (jit_break_name, nullptr, the_objfile);
+	= lookup_minimal_symbol_text (pspace, jit_break_name, &the_objfile);
       if (reg_symbol.minsym == NULL
 	  || reg_symbol.value_address () == 0)
 	{
 	  /* No need to repeat the lookup the next time.  */
-	  the_objfile->skip_jit_symbol_lookup = true;
+	  the_objfile.skip_jit_symbol_lookup = true;
 	  continue;
 	}
 
       bound_minimal_symbol desc_symbol
-	= lookup_minimal_symbol (jit_descriptor_name, NULL, the_objfile);
+	= lookup_minimal_symbol_linkage (jit_descriptor_name,
+					 &the_objfile, true);
       if (desc_symbol.minsym == NULL
 	  || desc_symbol.value_address () == 0)
 	{
 	  /* No need to repeat the lookup the next time.  */
-	  the_objfile->skip_jit_symbol_lookup = true;
+	  the_objfile.skip_jit_symbol_lookup = true;
 	  continue;
 	}
 
       jiter_objfile_data *objf_data
-	= get_jiter_objfile_data (the_objfile);
+	= get_jiter_objfile_data (&the_objfile);
       objf_data->register_code = reg_symbol.minsym;
       objf_data->descriptor = desc_symbol.minsym;
 
-      CORE_ADDR addr = objf_data->register_code->value_address (the_objfile);
+      CORE_ADDR addr = objf_data->register_code->value_address (&the_objfile);
       jit_debug_printf ("breakpoint_addr = %s", paddress (gdbarch, addr));
 
       /* Check if we need to re-create the breakpoint.  */
@@ -980,8 +983,10 @@ jit_unwind_reg_get_impl (struct gdb_unwind_callbacks *cb, int regnum)
   size = register_size (frame_arch, gdb_reg);
   value = ((struct gdb_reg_value *)
 	   xmalloc (sizeof (struct gdb_reg_value) + size - 1));
-  value->defined = deprecated_frame_register_read (priv->this_frame, gdb_reg,
-						   value->value);
+  value->defined
+    = deprecated_frame_register_read (priv->this_frame, gdb_reg,
+				      gdb::make_array_view (value->value,
+							    size));
   value->size = size;
   value->free = reg_value_free_impl;
   return value;
@@ -1007,7 +1012,7 @@ jit_dealloc_cache (frame_info *this_frame, void *cache)
 
 static int
 jit_frame_sniffer (const struct frame_unwind *self,
-		   frame_info_ptr this_frame, void **cache)
+		   const frame_info_ptr &this_frame, void **cache)
 {
   struct jit_unwind_private *priv_data;
   struct gdb_unwind_callbacks callbacks;
@@ -1053,7 +1058,7 @@ jit_frame_sniffer (const struct frame_unwind *self,
    the loaded plugin.  */
 
 static void
-jit_frame_this_id (frame_info_ptr this_frame, void **cache,
+jit_frame_this_id (const frame_info_ptr &this_frame, void **cache,
 		   struct frame_id *this_id)
 {
   struct jit_unwind_private priv;
@@ -1082,7 +1087,7 @@ jit_frame_this_id (frame_info_ptr this_frame, void **cache,
    the register from the cache.  */
 
 static struct value *
-jit_frame_prev_register (frame_info_ptr this_frame, void **cache, int reg)
+jit_frame_prev_register (const frame_info_ptr &this_frame, void **cache, int reg)
 {
   struct jit_unwind_private *priv = (struct jit_unwind_private *) *cache;
   struct gdbarch *gdbarch;
@@ -1091,7 +1096,7 @@ jit_frame_prev_register (frame_info_ptr this_frame, void **cache, int reg)
     return frame_unwind_got_optimized (this_frame, reg);
 
   gdbarch = priv->regcache->arch ();
-  gdb_byte *buf = (gdb_byte *) alloca (register_size (gdbarch, reg));
+  gdb::byte_vector buf (register_size (gdbarch, reg));
   enum register_status status = priv->regcache->cooked_read (reg, buf);
 
   if (status == REG_VALID)
@@ -1103,17 +1108,17 @@ jit_frame_prev_register (frame_info_ptr this_frame, void **cache, int reg)
 /* Relay everything back to the unwinder registered by the JIT debug
    info reader.*/
 
-static const struct frame_unwind jit_frame_unwind =
-{
+static const struct frame_unwind_legacy jit_frame_unwind (
   "jit",
   NORMAL_FRAME,
+  FRAME_UNWIND_EXTENSION,
   default_frame_unwind_stop_reason,
   jit_frame_this_id,
   jit_frame_prev_register,
   NULL,
   jit_frame_sniffer,
   jit_dealloc_cache
-};
+);
 
 
 /* This is the information that is stored at jit_gdbarch_data for each
@@ -1147,7 +1152,10 @@ jit_prepend_unwinder (struct gdbarch *gdbarch)
     }
 }
 
-/* Register any already created translations.  */
+/* Looks for the descriptor and registration symbols and breakpoints
+   the registration function.  If it finds both, it registers all the
+   already JITed code.  If it has already found the symbols, then it
+   doesn't try again.  */
 
 static void
 jit_inferior_init (inferior *inf)
@@ -1155,7 +1163,7 @@ jit_inferior_init (inferior *inf)
   struct jit_descriptor descriptor;
   struct jit_code_entry cur_entry;
   CORE_ADDR cur_entry_addr;
-  struct gdbarch *gdbarch = inf->gdbarch;
+  struct gdbarch *gdbarch = inf->arch ();
   program_space *pspace = inf->pspace;
 
   jit_debug_printf ("called");
@@ -1164,14 +1172,14 @@ jit_inferior_init (inferior *inf)
 
   jit_breakpoint_re_set_internal (gdbarch, pspace);
 
-  for (objfile *jiter : pspace->objfiles ())
+  for (objfile &jiter : pspace->objfiles ())
     {
-      if (jiter->jiter_data == nullptr)
+      if (jiter.jiter_data == nullptr)
 	continue;
 
       /* Read the descriptor so we can check the version number and load
 	 any already JITed functions.  */
-      if (!jit_read_descriptor (gdbarch, &descriptor, jiter))
+      if (!jit_read_descriptor (gdbarch, &descriptor, &jiter))
 	continue;
 
       /* Check that the version number agrees with that we support.  */
@@ -1203,10 +1211,7 @@ jit_inferior_init (inferior *inf)
     }
 }
 
-/* Looks for the descriptor and registration symbols and breakpoints
-   the registration function.  If it finds both, it registers all the
-   already JITed code.  If it has already found the symbols, then it
-   doesn't try again.  */
+/* inferior_created observer.  */
 
 static void
 jit_inferior_created_hook (inferior *inf)
@@ -1214,13 +1219,22 @@ jit_inferior_created_hook (inferior *inf)
   jit_inferior_init (inf);
 }
 
+/* inferior_execd observer.  */
+
+static void
+jit_inferior_execd_hook (inferior *exec_inf, inferior *follow_inf)
+{
+  jit_inferior_init (follow_inf);
+}
+
 /* Exported routine to call to re-set the jit breakpoints,
    e.g. when a program is rerun.  */
 
 void
 jit_breakpoint_re_set (void)
 {
-  jit_breakpoint_re_set_internal (target_gdbarch (), current_program_space);
+  jit_breakpoint_re_set_internal (current_inferior ()->arch (),
+				  current_program_space);
 }
 
 /* This function cleans up any code entries left over when the
@@ -1230,10 +1244,10 @@ jit_breakpoint_re_set (void)
 static void
 jit_inferior_exit_hook (struct inferior *inf)
 {
-  for (objfile *objf : current_program_space->objfiles_safe ())
+  for (objfile &objf : current_program_space->objfiles_safe ())
     {
-      if (objf->jited_data != nullptr && objf->jited_data->addr != 0)
-	objf->unlink ();
+      if (objf.jited_data != nullptr && objf.jited_data->addr != 0)
+	objf.unlink ();
     }
 }
 
@@ -1285,9 +1299,17 @@ jit_event_handler (gdbarch *gdbarch, objfile *jiter)
     }
 }
 
-void _initialize_jit ();
-void
-_initialize_jit ()
+/* Implementation of "show jit-reader-directory".  */
+
+static void
+show_jit_reader_directory (const char *args, int from_tty)
+{
+  gdb_printf (_("JIT reader directory is %ps.\n"),
+	      styled_string (file_name_style.style (),
+			     jit_reader_dir.c_str ()));
+}
+
+INIT_GDB_FILE (jit)
 {
   jit_reader_dir = relocate_gdb_directory (JIT_READER_DIR,
 					   JIT_READER_DIR_RELOCATABLE);
@@ -1304,7 +1326,7 @@ _initialize_jit ()
 	   &maintenanceinfolist);
 
   gdb::observers::inferior_created.attach (jit_inferior_created_hook, "jit");
-  gdb::observers::inferior_execd.attach (jit_inferior_created_hook, "jit");
+  gdb::observers::inferior_execd.attach (jit_inferior_execd_hook, "jit");
   gdb::observers::inferior_exit.attach (jit_inferior_exit_hook, "jit");
   gdb::observers::breakpoint_deleted.attach (jit_breakpoint_deleted, "jit");
 
@@ -1316,9 +1338,9 @@ _initialize_jit ()
 Load FILE as debug info reader and unwinder for JIT compiled code.\n\
 Usage: jit-reader-load FILE\n\
 Try to load file FILE as a debug info reader (and unwinder) for\n\
-JIT compiled code.  The file is loaded from " JIT_READER_DIR ",\n\
-relocated relative to the GDB executable if required."));
-      set_cmd_completer (c, filename_completer);
+JIT compiled code.  If FILE is not an absolute file name, it is found\n\
+relative to a built-in directory.  See \"show jit-reader-directory\"."));
+      set_cmd_completer (c, deprecated_filename_completer);
 
       c = add_com ("jit-reader-unload", no_class,
 		   jit_reader_unload_command, _("\
@@ -1326,5 +1348,11 @@ Unload the currently loaded JIT debug info reader.\n\
 Usage: jit-reader-unload\n\n\
 Do \"help jit-reader-load\" for info on loading debug info readers."));
       set_cmd_completer (c, noop_completer);
+
+      add_cmd ("jit-reader-directory", class_obscure,
+	       show_jit_reader_directory,
+	       _("Show the JIT reader directory.\n\
+This is the directory used by \"jit-reader-load\" when given\n\
+a relative file name."), &showlist);
     }
 }
