@@ -1,6 +1,6 @@
 /* Memory attributes support, for GDB.
 
-   Copyright (C) 2001-2021 Free Software Foundation, Inc.
+   Copyright (C) 2001-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,9 +17,9 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "command.h"
-#include "gdbcmd.h"
+#include "cli/cli-cmds.h"
+#include "cli/cli-style.h"
 #include "memattr.h"
 #include "target.h"
 #include "target-dcache.h"
@@ -29,6 +29,8 @@
 #include "cli/cli-utils.h"
 #include <algorithm>
 #include "gdbarch.h"
+#include "inferior.h"
+#include "progspace.h"
 
 static std::vector<mem_region> user_mem_region_list, target_mem_region_list;
 static std::vector<mem_region> *mem_region_list = &target_mem_region_list;
@@ -60,11 +62,11 @@ show_inaccessible_by_default (struct ui_file *file, int from_tty,
 			      const char *value)
 {
   if (inaccessible_by_default)
-    fprintf_filtered (file, _("Unknown memory addresses will "
-			      "be treated as inaccessible.\n"));
+    gdb_printf (file, _("Unknown memory addresses will "
+			"be treated as inaccessible.\n"));
   else
-    fprintf_filtered (file, _("Unknown memory addresses "
-			      "will be treated as RAM.\n"));          
+    gdb_printf (file, _("Unknown memory addresses "
+			"will be treated as RAM.\n"));
 }
 
 /* This function should be called before any command which would
@@ -90,7 +92,8 @@ require_user_regions (int from_tty)
   /* Otherwise, let the user know how to get back.  */
   if (from_tty)
     warning (_("Switching to manual control of memory regions; use "
-	       "\"mem auto\" to fetch regions from the target again."));
+	       "\"%ps\" to fetch regions from the target again."),
+	     styled_string (command_style.style (), "mem auto"));
 
   /* And create a new list (copy of the target-supplied regions) for the user
      to modify.  */
@@ -121,7 +124,7 @@ create_user_mem_region (CORE_ADDR lo, CORE_ADDR hi,
   /* lo == hi is a useless empty region.  */
   if (lo >= hi && hi != 0)
     {
-      printf_unfiltered (_("invalid memory region: low >= high\n"));
+      gdb_printf (_("invalid memory region: low >= high\n"));
       return;
     }
 
@@ -133,7 +136,7 @@ create_user_mem_region (CORE_ADDR lo, CORE_ADDR hi,
   int ix = std::distance (user_mem_region_list.begin (), it);
 
   /* Check for an overlapping memory region.  We only need to check
-     in the vincinity - at most one before and one after the
+     in the vicinity - at most one before and one after the
      insertion point.  */
   for (int i = ix - 1; i < ix + 1; i++)
     {
@@ -148,7 +151,7 @@ create_user_mem_region (CORE_ADDR lo, CORE_ADDR hi,
 	  || (hi > n.lo && (hi <= n.hi || n.hi == 0))
 	  || (lo <= n.lo && ((hi >= n.hi && n.hi != 0) || hi == 0)))
 	{
-	  printf_unfiltered (_("overlapping memory region\n"));
+	  gdb_printf (_("overlapping memory region\n"));
 	  return;
 	}
     }
@@ -181,7 +184,7 @@ lookup_mem_region (CORE_ADDR addr)
 
   /* Either find memory range containing ADDR, or set LO and HI
      to the nearest boundaries of an existing memory range.
-     
+
      If we ever want to support a huge list of memory regions, this
      check should be replaced with a binary search (probably using
      VEC_lower_bound).  */
@@ -213,8 +216,8 @@ lookup_mem_region (CORE_ADDR addr)
   region.lo = lo;
   region.hi = hi;
 
-  /* When no memory map is defined at all, we always return 
-     'default_mem_attrib', so that we do not make all memory 
+  /* When no memory map is defined at all, we always return
+     'default_mem_attrib', so that we do not make all memory
      inaccessible for targets that don't provide a memory map.  */
   if (inaccessible_by_default && !mem_region_list->empty ())
     region.attrib = mem_attrib::unknown ();
@@ -339,44 +342,44 @@ static void
 info_mem_command (const char *args, int from_tty)
 {
   if (mem_use_target ())
-    printf_filtered (_("Using memory regions provided by the target.\n"));
+    gdb_printf (_("Using memory regions provided by the target.\n"));
   else
-    printf_filtered (_("Using user-defined memory regions.\n"));
+    gdb_printf (_("Using user-defined memory regions.\n"));
 
   require_target_regions ();
 
   if (mem_region_list->empty ())
     {
-      printf_unfiltered (_("There are no memory regions defined.\n"));
+      gdb_printf (_("There are no memory regions defined.\n"));
       return;
     }
 
-  printf_filtered ("Num ");
-  printf_filtered ("Enb ");
-  printf_filtered ("Low Addr   ");
-  if (gdbarch_addr_bit (target_gdbarch ()) > 32)
-    printf_filtered ("        ");
-  printf_filtered ("High Addr  ");
-  if (gdbarch_addr_bit (target_gdbarch ()) > 32)
-    printf_filtered ("        ");
-  printf_filtered ("Attrs ");
-  printf_filtered ("\n");
+  gdb_printf ("Num ");
+  gdb_printf ("Enb ");
+  gdb_printf ("Low Addr   ");
+  if (gdbarch_addr_bit (current_inferior ()->arch ()) > 32)
+    gdb_printf ("        ");
+  gdb_printf ("High Addr  ");
+  if (gdbarch_addr_bit (current_inferior ()->arch ()) > 32)
+    gdb_printf ("        ");
+  gdb_printf ("Attrs ");
+  gdb_printf ("\n");
 
   for (const mem_region &m : *mem_region_list)
     {
       const char *tmp;
 
-      printf_filtered ("%-3d %-3c\t",
-		       m.number,
-		       m.enabled_p ? 'y' : 'n');
-      if (gdbarch_addr_bit (target_gdbarch ()) <= 32)
+      gdb_printf ("%-3d %-3c\t",
+		  m.number,
+		  m.enabled_p ? 'y' : 'n');
+      if (gdbarch_addr_bit (current_inferior ()->arch ()) <= 32)
 	tmp = hex_string_custom (m.lo, 8);
       else
 	tmp = hex_string_custom (m.lo, 16);
-      
-      printf_filtered ("%s ", tmp);
 
-      if (gdbarch_addr_bit (target_gdbarch ()) <= 32)
+      gdb_printf ("%s ", tmp);
+
+      if (gdbarch_addr_bit (current_inferior ()->arch ()) <= 32)
 	{
 	  if (m.hi == 0)
 	    tmp = "0x100000000";
@@ -391,7 +394,7 @@ info_mem_command (const char *args, int from_tty)
 	    tmp = hex_string_custom (m.hi, 16);
 	}
 
-      printf_filtered ("%s ", tmp);
+      gdb_printf ("%s ", tmp);
 
       /* Print a token for each attribute.
 
@@ -408,32 +411,32 @@ info_mem_command (const char *args, int from_tty)
       switch (m.attrib.mode)
 	{
 	case MEM_RW:
-	  printf_filtered ("rw ");
+	  gdb_printf ("rw ");
 	  break;
 	case MEM_RO:
-	  printf_filtered ("ro ");
+	  gdb_printf ("ro ");
 	  break;
 	case MEM_WO:
-	  printf_filtered ("wo ");
+	  gdb_printf ("wo ");
 	  break;
 	case MEM_FLASH:
-	  printf_filtered ("flash blocksize 0x%x ", m.attrib.blocksize);
+	  gdb_printf ("flash blocksize 0x%x ", m.attrib.blocksize);
 	  break;
 	}
 
       switch (m.attrib.width)
 	{
 	case MEM_WIDTH_8:
-	  printf_filtered ("8 ");
+	  gdb_printf ("8 ");
 	  break;
 	case MEM_WIDTH_16:
-	  printf_filtered ("16 ");
+	  gdb_printf ("16 ");
 	  break;
 	case MEM_WIDTH_32:
-	  printf_filtered ("32 ");
+	  gdb_printf ("32 ");
 	  break;
 	case MEM_WIDTH_64:
-	  printf_filtered ("64 ");
+	  gdb_printf ("64 ");
 	  break;
 	case MEM_WIDTH_UNSPECIFIED:
 	  break;
@@ -441,24 +444,24 @@ info_mem_command (const char *args, int from_tty)
 
 #if 0
       if (attrib->hwbreak)
-	printf_filtered ("hwbreak");
+	gdb_printf ("hwbreak");
       else
-	printf_filtered ("swbreak");
+	gdb_printf ("swbreak");
 #endif
 
       if (m.attrib.cache)
-	printf_filtered ("cache ");
+	gdb_printf ("cache ");
       else
-	printf_filtered ("nocache ");
+	gdb_printf ("nocache ");
 
 #if 0
       if (attrib->verify)
-	printf_filtered ("verify ");
+	gdb_printf ("verify ");
       else
-	printf_filtered ("noverify ");
+	gdb_printf ("noverify ");
 #endif
 
-      printf_filtered ("\n");
+      gdb_printf ("\n");
     }
 }
 
@@ -474,7 +477,7 @@ mem_enable (int num)
 	m.enabled_p = 1;
 	return;
       }
-  printf_unfiltered (_("No memory region number %d.\n"), num);
+  gdb_printf (_("No memory region number %d.\n"), num);
 }
 
 static void
@@ -482,7 +485,7 @@ enable_mem_command (const char *args, int from_tty)
 {
   require_user_regions (from_tty);
 
-  target_dcache_invalidate ();
+  target_dcache_invalidate (current_program_space->aspace);
 
   if (args == NULL || *args == '\0')
     { /* Enable all mem regions.  */
@@ -512,7 +515,7 @@ mem_disable (int num)
 	m.enabled_p = 0;
 	return;
       }
-  printf_unfiltered (_("No memory region number %d.\n"), num);
+  gdb_printf (_("No memory region number %d.\n"), num);
 }
 
 static void
@@ -520,7 +523,7 @@ disable_mem_command (const char *args, int from_tty)
 {
   require_user_regions (from_tty);
 
-  target_dcache_invalidate ();
+  target_dcache_invalidate (current_program_space->aspace);
 
   if (args == NULL || *args == '\0')
     {
@@ -545,7 +548,7 @@ mem_delete (int num)
 {
   if (!mem_region_list)
     {
-      printf_unfiltered (_("No memory region number %d.\n"), num);
+      gdb_printf (_("No memory region number %d.\n"), num);
       return;
     }
 
@@ -558,7 +561,7 @@ mem_delete (int num)
   if (it != mem_region_list->end ())
     mem_region_list->erase (it);
   else
-    printf_unfiltered (_("No memory region number %d.\n"), num);
+    gdb_printf (_("No memory region number %d.\n"), num);
 }
 
 static void
@@ -566,7 +569,7 @@ delete_mem_command (const char *args, int from_tty)
 {
   require_user_regions (from_tty);
 
-  target_dcache_invalidate ();
+  target_dcache_invalidate (current_program_space->aspace);
 
   if (args == NULL || *args == '\0')
     {
@@ -589,13 +592,10 @@ delete_mem_command (const char *args, int from_tty)
 static struct cmd_list_element *mem_set_cmdlist;
 static struct cmd_list_element *mem_show_cmdlist;
 
-void _initialize_mem ();
-void
-_initialize_mem ()
+INIT_GDB_FILE (mem)
 {
   add_com ("mem", class_vars, mem_command, _("\
-Define attributes for memory region or reset memory region handling to "
-"target-based.\n\
+Define or reset attributes for memory regions.\n\
 Usage: mem auto\n\
        mem LOW HIGH [MODE WIDTH CACHE],\n\
 where MODE  may be rw (read/write), ro (read-only) or wo (write-only),\n\
@@ -623,14 +623,11 @@ Do \"info mem\" to see current list of IDs."), &deletelist);
   add_info ("mem", info_mem_command,
 	    _("Memory region attributes."));
 
-  add_basic_prefix_cmd ("mem", class_vars, _("\
-Memory regions settings."),
-			&mem_set_cmdlist,
-			0/* allow-unknown */, &setlist);
-  add_show_prefix_cmd ("mem", class_vars, _("\
-Memory regions settings."),
-		       &mem_show_cmdlist,
-		       0/* allow-unknown */, &showlist);
+  add_setshow_prefix_cmd ("mem", class_vars,
+			  _("Memory regions settings."),
+			  _("Memory regions settings."),
+			  &mem_set_cmdlist, &mem_show_cmdlist,
+			  &setlist, &showlist);
 
   add_setshow_boolean_cmd ("inaccessible-by-default", no_class,
 				  &inaccessible_by_default, _("\
