@@ -1,6 +1,6 @@
 /* Target-dependent code for FreeBSD/mips.
 
-   Copyright (C) 2017-2022 Free Software Foundation, Inc.
+   Copyright (C) 2017-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,7 +17,7 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
+#include "extract-store-integer.h"
 #include "osabi.h"
 #include "regset.h"
 #include "trad-frame.h"
@@ -275,7 +275,7 @@ mips_fbsd_iterate_over_regset_sections (struct gdbarch *gdbarch,
 
 static void
 mips_fbsd_sigframe_init (const struct tramp_frame *self,
-			 frame_info_ptr this_frame,
+			 const frame_info_ptr &this_frame,
 			 struct trad_frame_cache *cache,
 			 CORE_ADDR func)
 {
@@ -367,7 +367,7 @@ static const struct tramp_frame mips_fbsd_sigframe =
 
 static void
 mips64_fbsd_sigframe_init (const struct tramp_frame *self,
-			   frame_info_ptr this_frame,
+			   const frame_info_ptr &this_frame,
 			   struct trad_frame_cache *cache,
 			   CORE_ADDR func)
 {
@@ -468,20 +468,38 @@ static const struct tramp_frame mips64_fbsd_sigframe =
 static CORE_ADDR
 mips_fbsd_skip_solib_resolver (struct gdbarch *gdbarch, CORE_ADDR pc)
 {
-  struct bound_minimal_symbol msym
-    = lookup_bound_minimal_symbol ("_mips_rtld_bind");
+  bound_minimal_symbol msym
+    = lookup_minimal_symbol (current_program_space, "_mips_rtld_bind");
   if (msym.minsym != nullptr && msym.value_address () == pc)
     return frame_unwind_caller_pc (get_current_frame ());
 
   return fbsd_skip_solib_resolver (gdbarch, pc);
 }
 
-/* FreeBSD/mips uses a slightly different `struct link_map' than the
-   other FreeBSD platforms as it includes an additional `l_off'
-   member.  */
+/* solib_ops for ILP32 FreeBSD/MIPS systems.  */
 
-static struct link_map_offsets *
-mips_fbsd_ilp32_fetch_link_map_offsets (void)
+struct mips_fbsd_ilp32_solib_ops : public svr4_solib_ops
+{
+  using svr4_solib_ops::svr4_solib_ops;
+
+  /* FreeBSD/MIPS uses a slightly different `struct link_map' than the
+     other FreeBSD platforms as it includes an additional `l_off' member.  */
+
+  link_map_offsets *fetch_link_map_offsets () const override;
+};
+
+/* Return a new solib_ops for ILP32 FreeBSD/MIPS systems.  */
+
+static solib_ops_up
+make_mips_fbsd_ilp32_solib_ops (program_space *pspace)
+{
+  return std::make_unique<mips_fbsd_ilp32_solib_ops> (pspace);
+}
+
+/* See mips_fbsd_ilp32_solib_ops.  */
+
+link_map_offsets *
+mips_fbsd_ilp32_solib_ops::fetch_link_map_offsets () const
 {
   static struct link_map_offsets lmo;
   static struct link_map_offsets *lmp = NULL;
@@ -508,8 +526,30 @@ mips_fbsd_ilp32_fetch_link_map_offsets (void)
   return lmp;
 }
 
-static struct link_map_offsets *
-mips_fbsd_lp64_fetch_link_map_offsets (void)
+/* solib_ops for LP64 FreeBSD/MIPS systems.  */
+
+struct mips_fbsd_lp64_solib_ops : public svr4_solib_ops
+{
+  using svr4_solib_ops::svr4_solib_ops;
+
+  /* FreeBSD/MIPS uses a slightly different `struct link_map' than the
+     other FreeBSD platforms as it includes an additional `l_off' member.  */
+
+  link_map_offsets *fetch_link_map_offsets () const override;
+};
+
+/* Return a new solib_ops for LP64 FreeBSD/MIPS systems.  */
+
+static solib_ops_up
+make_mips_fbsd_lp64_solib_ops (program_space *pspace)
+{
+  return std::make_unique<mips_fbsd_lp64_solib_ops> (pspace);
+}
+
+/* See mips_fbsd_lp64_solib_ops.  */
+
+link_map_offsets *
+mips_fbsd_lp64_solib_ops::fetch_link_map_offsets () const
 {
   static struct link_map_offsets lmo;
   static struct link_map_offsets *lmp = NULL;
@@ -544,7 +584,7 @@ mips_fbsd_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
   /* Generic FreeBSD support.  */
   fbsd_init_abi (info, gdbarch);
 
-  set_gdbarch_software_single_step (gdbarch, mips_software_single_step);
+  set_gdbarch_get_next_pcs (gdbarch, mips_software_single_step);
 
   switch (abi)
     {
@@ -565,15 +605,12 @@ mips_fbsd_init_abi (struct gdbarch_info info, struct gdbarch *gdbarch)
   set_gdbarch_skip_solib_resolver (gdbarch, mips_fbsd_skip_solib_resolver);
 
   /* FreeBSD/mips has SVR4-style shared libraries.  */
-  set_solib_svr4_fetch_link_map_offsets
-    (gdbarch, (gdbarch_ptr_bit (gdbarch) == 32 ?
-	       mips_fbsd_ilp32_fetch_link_map_offsets :
-	       mips_fbsd_lp64_fetch_link_map_offsets));
+  set_solib_svr4_ops (gdbarch, (gdbarch_ptr_bit (gdbarch) == 32
+				? make_mips_fbsd_ilp32_solib_ops
+				: make_mips_fbsd_lp64_solib_ops));
 }
 
-void _initialize_mips_fbsd_tdep ();
-void
-_initialize_mips_fbsd_tdep ()
+INIT_GDB_FILE (mips_fbsd_tdep)
 {
   gdbarch_register_osabi (bfd_arch_mips, 0, GDB_OSABI_FREEBSD,
 			  mips_fbsd_init_abi);
