@@ -1,6 +1,6 @@
 /* Definitions for targets which report shared library events.
 
-   Copyright (C) 2007-2022 Free Software Foundation, Inc.
+   Copyright (C) 2007-2026 Free Software Foundation, Inc.
 
    This file is part of GDB.
 
@@ -17,9 +17,8 @@
    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
-#include "defs.h"
 #include "objfiles.h"
-#include "solist.h"
+#include "solib.h"
 #include "symtab.h"
 #include "symfile.h"
 #include "target.h"
@@ -28,12 +27,8 @@
 #include "inferior.h"
 
 /* Private data for each loaded library.  */
-struct lm_info_target : public lm_info_base
+struct lm_info_target final : public lm_info
 {
-  /* The library's name.  The name is normally kept in the struct
-     so_list; it is only here during XML parsing.  */
-  std::string name;
-
   /* The target can either specify segment bases or section bases, not
      both.  */
 
@@ -47,14 +42,23 @@ struct lm_info_target : public lm_info_base
 
   /* The cached offsets for each section of this shared library,
      determined from SEGMENT_BASES, or SECTION_BASES.  */
-  section_offsets offsets;
+  std::vector<CORE_ADDR> offsets;
 };
 
-typedef std::vector<std::unique_ptr<lm_info_target>> lm_info_vector;
+using lm_info_target_up = std::unique_ptr<lm_info_target>;
+
+/* Type used to convey the information about one target library while parsing
+   the XML.  */
+
+struct target_library
+{
+  std::string name;
+  lm_info_target_up info;
+};
 
 #if !defined(HAVE_LIBEXPAT)
 
-static lm_info_vector
+static std::vector<target_library>
 solib_target_parse_libraries (const char *library)
 {
   static int have_warned;
@@ -66,7 +70,7 @@ solib_target_parse_libraries (const char *library)
 		 "at compile time"));
     }
 
-  return lm_info_vector ();
+  return {};
 }
 
 #else /* HAVE_LIBEXPAT */
@@ -81,17 +85,17 @@ library_list_start_segment (struct gdb_xml_parser *parser,
 			    void *user_data,
 			    std::vector<gdb_xml_value> &attributes)
 {
-  lm_info_vector *list = (lm_info_vector *) user_data;
-  lm_info_target *last = list->back ().get ();
+  const auto list = static_cast<std::vector<target_library> *> (user_data);
+  target_library &last = list->back ();
   ULONGEST *address_p
     = (ULONGEST *) xml_find_attribute (attributes, "address")->value.get ();
   CORE_ADDR address = (CORE_ADDR) *address_p;
 
-  if (!last->section_bases.empty ())
+  if (!last.info->section_bases.empty ())
     gdb_xml_error (parser,
 		   _("Library list with both segments and sections"));
 
-  last->segment_bases.push_back (address);
+  last.info->segment_bases.push_back (address);
 }
 
 static void
@@ -100,17 +104,17 @@ library_list_start_section (struct gdb_xml_parser *parser,
 			    void *user_data,
 			    std::vector<gdb_xml_value> &attributes)
 {
-  lm_info_vector *list = (lm_info_vector *) user_data;
-  lm_info_target *last = list->back ().get ();
+  const auto list = static_cast<std::vector<target_library> *> (user_data);
+  target_library &last = list->back ();
   ULONGEST *address_p
     = (ULONGEST *) xml_find_attribute (attributes, "address")->value.get ();
   CORE_ADDR address = (CORE_ADDR) *address_p;
 
-  if (!last->segment_bases.empty ())
+  if (!last.info->segment_bases.empty ())
     gdb_xml_error (parser,
 		   _("Library list with both segments and sections"));
 
-  last->section_bases.push_back (address);
+  last.info->section_bases.push_back (address);
 }
 
 /* Handle the start of a <library> element.  */
@@ -121,12 +125,12 @@ library_list_start_library (struct gdb_xml_parser *parser,
 			    void *user_data,
 			    std::vector<gdb_xml_value> &attributes)
 {
-  lm_info_vector *list = (lm_info_vector *) user_data;
-  lm_info_target *item = new lm_info_target;
-  item->name
+  const auto list = static_cast<std::vector<target_library> *> (user_data);
+  std::string name
     = (const char *) xml_find_attribute (attributes, "name")->value.get ();
 
-  list->emplace_back (item);
+  list->emplace_back (target_library { std::move (name),
+				       std::make_unique<lm_info_target> () });
 }
 
 static void
@@ -134,10 +138,10 @@ library_list_end_library (struct gdb_xml_parser *parser,
 			  const struct gdb_xml_element *element,
 			  void *user_data, const char *body_text)
 {
-  lm_info_vector *list = (lm_info_vector *) user_data;
-  lm_info_target *lm_info = list->back ().get ();
+  const auto list = static_cast<std::vector<target_library> *> (user_data);
+  target_library &last = list->back ();
 
-  if (lm_info->segment_bases.empty () && lm_info->section_bases.empty ())
+  if (last.info->segment_bases.empty () && last.info->section_bases.empty ())
     gdb_xml_error (parser, _("No segment or section bases defined"));
 }
 
@@ -210,10 +214,10 @@ static const struct gdb_xml_element library_list_elements[] = {
   { NULL, NULL, NULL, GDB_XML_EF_NONE, NULL, NULL }
 };
 
-static lm_info_vector
+static std::vector<target_library>
 solib_target_parse_libraries (const char *library)
 {
-  lm_info_vector result;
+  std::vector<target_library> result;
 
   if (gdb_xml_parse_quick (_("target library list"), "library-list.dtd",
 			   library_list_elements, library, &result) == 0)
@@ -227,88 +231,42 @@ solib_target_parse_libraries (const char *library)
 }
 #endif
 
-static struct so_list *
-solib_target_current_sos (void)
+owning_intrusive_list<solib>
+target_solib_ops::current_sos () const
 {
-  struct so_list *new_solib, *start = NULL, *last = NULL;
+  owning_intrusive_list<solib> sos;
 
   /* Fetch the list of shared libraries.  */
-  gdb::optional<gdb::char_vector> library_document
+  std::optional<gdb::char_vector> library_document
     = target_read_stralloc (current_inferior ()->top_target (),
 			    TARGET_OBJECT_LIBRARIES, NULL);
   if (!library_document)
-    return NULL;
+    return {};
 
   /* Parse the list.  */
-  lm_info_vector library_list
+  std::vector<target_library> library_list
     = solib_target_parse_libraries (library_document->data ());
 
-  if (library_list.empty ())
-    return NULL;
+  /* Build a struct solib for each entry on the list.  */
+  for (auto &library : library_list)
+    sos.emplace_back (std::move (library.info), library.name, library.name,
+		      *this);
 
-  /* Build a struct so_list for each entry on the list.  */
-  for (auto &&info : library_list)
-    {
-      new_solib = XCNEW (struct so_list);
-      strncpy (new_solib->so_name, info->name.c_str (),
-	       SO_NAME_MAX_PATH_SIZE - 1);
-      new_solib->so_name[SO_NAME_MAX_PATH_SIZE - 1] = '\0';
-      strncpy (new_solib->so_original_name, info->name.c_str (),
-	       SO_NAME_MAX_PATH_SIZE - 1);
-      new_solib->so_original_name[SO_NAME_MAX_PATH_SIZE - 1] = '\0';
-
-      /* We no longer need this copy of the name.  */
-      info->name.clear ();
-
-      new_solib->lm_info = info.release ();
-
-      /* Add it to the list.  */
-      if (!start)
-	last = start = new_solib;
-      else
-	{
-	  last->next = new_solib;
-	  last = new_solib;
-	}
-    }
-
-  return start;
+  return sos;
 }
 
-static void
-solib_target_solib_create_inferior_hook (int from_tty)
-{
-  /* Nothing needed.  */
-}
-
-static void
-solib_target_clear_solib (void)
-{
-  /* Nothing needed.  */
-}
-
-static void
-solib_target_free_so (struct so_list *so)
-{
-  lm_info_target *li = (lm_info_target *) so->lm_info;
-
-  gdb_assert (li->name.empty ());
-
-  delete li;
-}
-
-static void
-solib_target_relocate_section_addresses (struct so_list *so,
-					 struct target_section *sec)
+void
+target_solib_ops::relocate_section_addresses (solib &so,
+					      target_section *sec) const
 {
   CORE_ADDR offset;
-  lm_info_target *li = (lm_info_target *) so->lm_info;
+  auto *li = gdb::checked_static_cast<lm_info_target *> (so.lm_info.get ());
 
   /* Build the offset table only once per object file.  We can not do
      it any earlier, since we need to open the file first.  */
   if (li->offsets.empty ())
     {
-      int num_sections = gdb_bfd_count_sections (so->abfd);
+      int num_sections = gdb_bfd_count_sections (so.abfd.get ());
 
       li->offsets.assign (num_sections, 0);
 
@@ -318,7 +276,7 @@ solib_target_relocate_section_addresses (struct so_list *so,
 	  asection *sect;
 	  int num_alloc_sections = 0;
 
-	  for (i = 0, sect = so->abfd->sections;
+	  for (i = 0, sect = so.abfd->sections;
 	       sect != NULL;
 	       i++, sect = sect->next)
 	    if ((bfd_section_flags (sect) & SEC_ALLOC))
@@ -327,15 +285,15 @@ solib_target_relocate_section_addresses (struct so_list *so,
 	  if (num_alloc_sections != li->section_bases.size ())
 	    warning (_("\
 Could not relocate shared library \"%s\": wrong number of ALLOC sections"),
-		     so->so_name);
+		     so.name.c_str ());
 	  else
 	    {
 	      int bases_index = 0;
 	      int found_range = 0;
 
-	      so->addr_low = ~(CORE_ADDR) 0;
-	      so->addr_high = 0;
-	      for (i = 0, sect = so->abfd->sections;
+	      so.addr_low = ~(CORE_ADDR) 0;
+	      so.addr_high = 0;
+	      for (i = 0, sect = so.abfd->sections;
 		   sect != NULL;
 		   i++, sect = sect->next)
 		{
@@ -348,40 +306,40 @@ Could not relocate shared library \"%s\": wrong number of ALLOC sections"),
 		      low = li->section_bases[i];
 		      high = low + bfd_section_size (sect) - 1;
 
-		      if (low < so->addr_low)
-			so->addr_low = low;
-		      if (high > so->addr_high)
-			so->addr_high = high;
-		      gdb_assert (so->addr_low <= so->addr_high);
+		      if (low < so.addr_low)
+			so.addr_low = low;
+		      if (high > so.addr_high)
+			so.addr_high = high;
+		      gdb_assert (so.addr_low <= so.addr_high);
 		      found_range = 1;
 		    }
 		  li->offsets[i] = li->section_bases[bases_index];
 		  bases_index++;
 		}
 	      if (!found_range)
-		so->addr_low = so->addr_high = 0;
-	      gdb_assert (so->addr_low <= so->addr_high);
+		so.addr_low = so.addr_high = 0;
+	      gdb_assert (so.addr_low <= so.addr_high);
 	    }
 	}
       else if (!li->segment_bases.empty ())
 	{
 	  symfile_segment_data_up data
-	    = get_symfile_segment_data (so->abfd);
+	    = get_symfile_segment_data (so.abfd.get ());
 
 	  if (data == NULL)
 	    warning (_("\
-Could not relocate shared library \"%s\": no segments"), so->so_name);
+Could not relocate shared library \"%s\": no segments"), so.name.c_str ());
 	  else
 	    {
 	      ULONGEST orig_delta;
 	      int i;
 
-	      if (!symfile_map_offsets_to_segments (so->abfd, data.get (),
+	      if (!symfile_map_offsets_to_segments (so.abfd.get (), data.get (),
 						    li->offsets,
 						    li->segment_bases.size (),
 						    li->segment_bases.data ()))
 		warning (_("\
-Could not relocate shared library \"%s\": bad offsets"), so->so_name);
+Could not relocate shared library \"%s\": bad offsets"), so.name.c_str ());
 
 	      /* Find the range of addresses to report for this library in
 		 "info sharedlibrary".  Report any consecutive segments
@@ -403,11 +361,11 @@ Could not relocate shared library \"%s\": bad offsets"), so->so_name);
 		    break;
 		}
 
-	      so->addr_low = li->segment_bases[0];
-	      so->addr_high = (data->segments[i - 1].base
+	      so.addr_low = li->segment_bases[0];
+	      so.addr_high = (data->segments[i - 1].base
 			       + data->segments[i - 1].size
 			       + orig_delta);
-	      gdb_assert (so->addr_low <= so->addr_high);
+	      gdb_assert (so.addr_low <= so.addr_high);
 	    }
 	}
     }
@@ -418,16 +376,8 @@ Could not relocate shared library \"%s\": bad offsets"), so->so_name);
   sec->endaddr += offset;
 }
 
-static int
-solib_target_open_symbol_file_object (int from_tty)
-{
-  /* We can't locate the main symbol file based on the target's
-     knowledge; the user has to specify it.  */
-  return 0;
-}
-
-static int
-solib_target_in_dynsym_resolve_code (CORE_ADDR pc)
+bool
+target_solib_ops::in_dynsym_resolve_code (CORE_ADDR pc) const
 {
   /* We don't have a range of addresses for the dynamic linker; there
      may not be one in the program's address space.  So only report
@@ -435,15 +385,10 @@ solib_target_in_dynsym_resolve_code (CORE_ADDR pc)
   return in_plt_section (pc);
 }
 
-const struct target_so_ops solib_target_so_ops =
+/* See solib-target.h.  */
+
+solib_ops_up
+make_target_solib_ops (program_space *pspace)
 {
-  solib_target_relocate_section_addresses,
-  solib_target_free_so,
-  nullptr,
-  solib_target_clear_solib,
-  solib_target_solib_create_inferior_hook,
-  solib_target_current_sos,
-  solib_target_open_symbol_file_object,
-  solib_target_in_dynsym_resolve_code,
-  solib_bfd_open,
-};
+  return std::make_unique<target_solib_ops> (pspace);
+}
