@@ -1,6 +1,6 @@
 /* The IGEN simulator generator for GDB, the GNU Debugger.
 
-   Copyright 2002-2018 Free Software Foundation, Inc.
+   Copyright 2002-2026 Free Software Foundation, Inc.
 
    Contributed by Andrew Cagney.
 
@@ -28,7 +28,7 @@
 #include "ld-insn.h"
 
 static insn_word_entry *
-parse_insn_word (line_ref *line, char *string, int word_nr)
+parse_insn_word (const line_ref *line, char *string, int word_nr)
 {
   char *chp;
   insn_word_entry *word = ZALLOC (insn_word_entry);
@@ -201,9 +201,9 @@ parse_insn_word (line_ref *line, char *string, int word_nr)
 	{
 	  if (strlen_pos == 0)
 	    {
-	      /* when the length/pos field is omited, an integer field
+	      /* when the length/pos field is omitted, an integer field
 	         is always binary */
-	      unsigned64 val = 0;
+	      uint64_t val = 0;
 	      int i;
 	      for (i = 0; i < strlen_val; i++)
 		{
@@ -409,7 +409,7 @@ parse_insn_words (insn_entry * insn, char *formats)
       insn->word[i] = word;
   }
 
-  /* Go over all fields that have conditionals refering to other
+  /* Go over all fields that have conditionals referring to other
      fields.  Link the fields up.  Verify that the two fields have the
      same size. Verify that the two fields are different */
   {
@@ -442,13 +442,13 @@ parse_insn_words (insn_entry * insn, char *formats)
 				&& strcmp (refered_field->val_string,
 					   cond->string) == 0)
 			      {
-				/* found field being refered to by conditonal */
+				/* found field being referred to by conditonal */
 				cond->field = refered_field;
-				/* check refered to and this field are
+				/* check referred to and this field are
 				   the same size */
 				if (f->width != refered_field->width)
 				  error (insn->line,
-					 "Conditional `%s' of field `%s' should be of size %s\n",
+					 "Conditional `%s' of field `%s' should be of size %i\n",
 					 cond->string, f->val_string,
 					 refered_field->width);
 			      }
@@ -849,7 +849,7 @@ parse_insn_model_record (table *file,
       /* Find the corresponding master model record for each name so
          that they can be linked in. */
       int index;
-      char *name = "";
+      const char *name = "";
       while (1)
 	{
 	  name = filter_next (new_insn_model->names, name);
@@ -914,7 +914,7 @@ static table_entry *
 parse_macro_record (table *file, table_entry *record)
 {
 #if 1
-  error (record->line, "Macros are not implemented");
+  error (record->line, "Macros are not implemented\n");
 #else
   /* parse the define record */
   if (record->nr_fields < nr_define_fields)
@@ -938,7 +938,7 @@ parse_macro_record (table *file, table_entry *record)
 
 
 insn_table *
-load_insn_table (char *file_name, cache_entry *cache)
+load_insn_table (const char *file_name, cache_entry *cache)
 {
   table *file = table_open (file_name);
   table_entry *record = table_read (file);
@@ -1279,7 +1279,7 @@ load_insn_table (char *file_name, cache_entry *cache)
 
 
 void
-print_insn_words (lf *file, insn_entry * insn)
+print_insn_words (lf *file, const insn_entry *insn)
 {
   insn_word_entry *word = insn->words;
   if (word != NULL)
@@ -1359,10 +1359,10 @@ print_insn_words (lf *file, insn_entry * insn)
 
 void
 function_entry_traverse (lf *file,
-			 function_entry * functions,
+			 const function_entry *functions,
 			 function_entry_handler * handler, void *data)
 {
-  function_entry *function;
+  const function_entry *function;
   for (function = functions; function != NULL; function = function->next)
     {
       handler (file, function, data);
@@ -1371,10 +1371,10 @@ function_entry_traverse (lf *file,
 
 void
 insn_table_traverse_insn (lf *file,
-			  insn_table *isa,
+			  const insn_table *isa,
 			  insn_entry_handler * handler, void *data)
 {
-  insn_entry *insn;
+  const insn_entry *insn;
   for (insn = isa->insns; insn != NULL; insn = insn->next)
     {
       handler (file, isa, insn, data);
@@ -1384,9 +1384,11 @@ insn_table_traverse_insn (lf *file,
 
 static void
 dump_function_entry (lf *file,
-		     char *prefix, function_entry * entry, char *suffix)
+		     const char *prefix,
+		     const function_entry *entry,
+		     const char *suffix)
 {
-  lf_printf (file, "%s(function_entry *) 0x%lx", prefix, (long) entry);
+  lf_printf (file, "%s(function_entry *) %p", prefix, entry);
   if (entry != NULL)
     {
       dump_line_ref (file, "\n(line ", entry->line, ")");
@@ -1396,14 +1398,16 @@ dump_function_entry (lf *file,
       lf_printf (file, "\n(param \"%s\")", entry->param);
       dump_table_entry (file, "\n(code ", entry->code, ")");
       lf_printf (file, "\n(is_internal %d)", entry->is_internal);
-      lf_printf (file, "\n(next 0x%lx)", (long) entry->next);
+      lf_printf (file, "\n(next %p)", entry->next);
     }
   lf_printf (file, "%s", suffix);
 }
 
 static void
 dump_function_entries (lf *file,
-		       char *prefix, function_entry * entry, char *suffix)
+		       const char *prefix,
+		       const function_entry *entry,
+		       const char *suffix)
 {
   lf_printf (file, "%s", prefix);
   lf_indent (file, +1);
@@ -1433,9 +1437,12 @@ cache_entry_type_to_str (cache_entry_type type)
 }
 
 static void
-dump_cache_entry (lf *file, char *prefix, cache_entry *entry, char *suffix)
+dump_cache_entry (lf *file,
+		  const char *prefix,
+		  const cache_entry *entry,
+		  const char *suffix)
 {
-  lf_printf (file, "%s(cache_entry *) 0x%lx", prefix, (long) entry);
+  lf_printf (file, "%s(cache_entry *) %p", prefix, entry);
   if (entry != NULL)
     {
       dump_line_ref (file, "\n(line ", entry->line, ")");
@@ -1446,13 +1453,16 @@ dump_cache_entry (lf *file, char *prefix, cache_entry *entry, char *suffix)
       dump_filter (file, "\n(original_fields ", entry->original_fields, ")");
       lf_printf (file, "\n(type \"%s\")", entry->type);
       lf_printf (file, "\n(expression \"%s\")", entry->expression);
-      lf_printf (file, "\n(next 0x%lx)", (long) entry->next);
+      lf_printf (file, "\n(next %p)", entry->next);
     }
   lf_printf (file, "%s", suffix);
 }
 
 void
-dump_cache_entries (lf *file, char *prefix, cache_entry *entry, char *suffix)
+dump_cache_entries (lf *file,
+		    const char *prefix,
+		    const cache_entry *entry,
+		    const char *suffix)
 {
   lf_printf (file, "%s", prefix);
   lf_indent (file, +1);
@@ -1466,9 +1476,12 @@ dump_cache_entries (lf *file, char *prefix, cache_entry *entry, char *suffix)
 }
 
 static void
-dump_model_data (lf *file, char *prefix, model_data *entry, char *suffix)
+dump_model_data (lf *file,
+		 const char *prefix,
+		 const model_data *entry,
+		 const char *suffix)
 {
-  lf_printf (file, "%s(model_data *) 0x%lx", prefix, (long) entry);
+  lf_printf (file, "%s(model_data *) %p", prefix, entry);
   if (entry != NULL)
     {
       lf_indent (file, +1);
@@ -1476,14 +1489,17 @@ dump_model_data (lf *file, char *prefix, model_data *entry, char *suffix)
       dump_filter (file, "\n(flags ", entry->flags, ")");
       dump_table_entry (file, "\n(entry ", entry->entry, ")");
       dump_table_entry (file, "\n(code ", entry->code, ")");
-      lf_printf (file, "\n(next 0x%lx)", (long) entry->next);
+      lf_printf (file, "\n(next %p)", entry->next);
       lf_indent (file, -1);
     }
   lf_printf (file, "%s", prefix);
 }
 
 static void
-dump_model_datas (lf *file, char *prefix, model_data *entry, char *suffix)
+dump_model_datas (lf *file,
+		  const char *prefix,
+		  const model_data *entry,
+		  const char *suffix)
 {
   lf_printf (file, "%s", prefix);
   lf_indent (file, +1);
@@ -1497,9 +1513,12 @@ dump_model_datas (lf *file, char *prefix, model_data *entry, char *suffix)
 }
 
 static void
-dump_model_entry (lf *file, char *prefix, model_entry *entry, char *suffix)
+dump_model_entry (lf *file,
+		  const char *prefix,
+		  const model_entry *entry,
+		  const char *suffix)
 {
-  lf_printf (file, "%s(model_entry *) 0x%lx", prefix, (long) entry);
+  lf_printf (file, "%s(model_entry *) %p", prefix, entry);
   if (entry != NULL)
     {
       lf_indent (file, +1);
@@ -1508,14 +1527,17 @@ dump_model_entry (lf *file, char *prefix, model_entry *entry, char *suffix)
       lf_printf (file, "\n(name \"%s\")", entry->name);
       lf_printf (file, "\n(full_name \"%s\")", entry->full_name);
       lf_printf (file, "\n(unit_data \"%s\")", entry->unit_data);
-      lf_printf (file, "\n(next 0x%lx)", (long) entry->next);
+      lf_printf (file, "\n(next %p)", entry->next);
       lf_indent (file, -1);
     }
   lf_printf (file, "%s", prefix);
 }
 
 static void
-dump_model_entries (lf *file, char *prefix, model_entry *entry, char *suffix)
+dump_model_entries (lf *file,
+		    const char *prefix,
+		    const model_entry *entry,
+		    const char *suffix)
 {
   lf_printf (file, "%s", prefix);
   lf_indent (file, +1);
@@ -1530,9 +1552,12 @@ dump_model_entries (lf *file, char *prefix, model_entry *entry, char *suffix)
 
 
 static void
-dump_model_table (lf *file, char *prefix, model_table *entry, char *suffix)
+dump_model_table (lf *file,
+		  const char *prefix,
+		  const model_table *entry,
+		  const char *suffix)
 {
-  lf_printf (file, "%s(model_table *) 0x%lx", prefix, (long) entry);
+  lf_printf (file, "%s(model_table *) %p", prefix, entry);
   if (entry != NULL)
     {
       lf_indent (file, +1);
@@ -1573,10 +1598,12 @@ insn_field_type_to_str (insn_field_type type)
 
 void
 dump_insn_field (lf *file,
-		 char *prefix, insn_field_entry *field, char *suffix)
+		 const char *prefix,
+		 const insn_field_entry *field,
+		 const char *suffix)
 {
   char *sep = " ";
-  lf_printf (file, "%s(insn_field_entry *) 0x%lx", prefix, (long) field);
+  lf_printf (file, "%s(insn_field_entry *) %p", prefix, field);
   if (field != NULL)
     {
       lf_indent (file, +1);
@@ -1603,8 +1630,8 @@ dump_insn_field (lf *file,
 	  lf_printf (file, "%s(val \"%s\")", sep, field->val_string);
 	  break;
 	}
-      lf_printf (file, "%s(next 0x%lx)", sep, (long) field->next);
-      lf_printf (file, "%s(prev 0x%lx)", sep, (long) field->prev);
+      lf_printf (file, "%s(next %p)", sep, field->next);
+      lf_printf (file, "%s(prev %p)", sep, field->prev);
       lf_indent (file, -1);
     }
   lf_printf (file, "%s", suffix);
@@ -1612,26 +1639,28 @@ dump_insn_field (lf *file,
 
 void
 dump_insn_word_entry (lf *file,
-		      char *prefix, insn_word_entry *word, char *suffix)
+		      const char *prefix,
+		      const insn_word_entry *word,
+		      const char *suffix)
 {
-  lf_printf (file, "%s(insn_word_entry *) 0x%lx", prefix, (long) word);
+  lf_printf (file, "%s(insn_word_entry *) %p", prefix, word);
   if (word != NULL)
     {
       int i;
       insn_field_entry *field;
       lf_indent (file, +1);
-      lf_printf (file, "\n(first 0x%lx)", (long) word->first);
-      lf_printf (file, "\n(last 0x%lx)", (long) word->last);
+      lf_printf (file, "\n(first %p)", word->first);
+      lf_printf (file, "\n(last %p)", word->last);
       lf_printf (file, "\n(bit");
       for (i = 0; i < options.insn_bit_size; i++)
-	lf_printf (file, "\n ((value %d) (mask %d) (field 0x%lx))",
+	lf_printf (file, "\n ((value %d) (mask %d) (field %p))",
 		   word->bit[i]->value, word->bit[i]->mask,
-		   (long) word->bit[i]->field);
+		   word->bit[i]->field);
       lf_printf (file, ")");
       for (field = word->first; field != NULL; field = field->next)
 	dump_insn_field (file, "\n(", field, ")");
       dump_filter (file, "\n(field_names ", word->field_names, ")");
-      lf_printf (file, "\n(next 0x%lx)", (long) word->next);
+      lf_printf (file, "\n(next %p)", word->next);
       lf_indent (file, -1);
     }
   lf_printf (file, "%s", suffix);
@@ -1639,7 +1668,9 @@ dump_insn_word_entry (lf *file,
 
 static void
 dump_insn_word_entries (lf *file,
-			char *prefix, insn_word_entry *word, char *suffix)
+			const char *prefix,
+			const insn_word_entry *word,
+			const char *suffix)
 {
   lf_printf (file, "%s", prefix);
   while (word != NULL)
@@ -1652,9 +1683,11 @@ dump_insn_word_entries (lf *file,
 
 static void
 dump_insn_model_entry (lf *file,
-		       char *prefix, insn_model_entry *model, char *suffix)
+		       const char *prefix,
+		       const insn_model_entry *model,
+		       const char *suffix)
 {
-  lf_printf (file, "%s(insn_model_entry *) 0x%lx", prefix, (long) model);
+  lf_printf (file, "%s(insn_model_entry *) %p", prefix, model);
   if (model != NULL)
     {
       lf_indent (file, +1);
@@ -1662,9 +1695,8 @@ dump_insn_model_entry (lf *file,
       dump_filter (file, "\n(names ", model->names, ")");
       lf_printf (file, "\n(full_name \"%s\")", model->full_name);
       lf_printf (file, "\n(unit_data \"%s\")", model->unit_data);
-      lf_printf (file, "\n(insn (insn_entry *) 0x%lx)", (long) model->insn);
-      lf_printf (file, "\n(next (insn_model_entry *) 0x%lx)",
-		 (long) model->next);
+      lf_printf (file, "\n(insn (insn_entry *) %p)", model->insn);
+      lf_printf (file, "\n(next (insn_model_entry *) %p)", model->next);
       lf_indent (file, -1);
     }
   lf_printf (file, "%s", suffix);
@@ -1672,7 +1704,9 @@ dump_insn_model_entry (lf *file,
 
 static void
 dump_insn_model_entries (lf *file,
-			 char *prefix, insn_model_entry *model, char *suffix)
+			 const char *prefix,
+			 const insn_model_entry *model,
+			 const char *suffix)
 {
   lf_printf (file, "%s", prefix);
   while (model != NULL)
@@ -1686,21 +1720,19 @@ dump_insn_model_entries (lf *file,
 
 static void
 dump_insn_mnemonic_entry (lf *file,
-			  char *prefix,
-			  insn_mnemonic_entry *mnemonic, char *suffix)
+			  const char *prefix,
+			  const insn_mnemonic_entry *mnemonic,
+			  const char *suffix)
 {
-  lf_printf (file, "%s(insn_mnemonic_entry *) 0x%lx", prefix,
-	     (long) mnemonic);
+  lf_printf (file, "%s(insn_mnemonic_entry *) %p", prefix, mnemonic);
   if (mnemonic != NULL)
     {
       lf_indent (file, +1);
       dump_line_ref (file, "\n(line ", mnemonic->line, ")");
       lf_printf (file, "\n(format \"%s\")", mnemonic->format);
       lf_printf (file, "\n(condition \"%s\")", mnemonic->condition);
-      lf_printf (file, "\n(insn (insn_entry *) 0x%lx)",
-		 (long) mnemonic->insn);
-      lf_printf (file, "\n(next (insn_mnemonic_entry *) 0x%lx)",
-		 (long) mnemonic->next);
+      lf_printf (file, "\n(insn (insn_entry *) %p)", mnemonic->insn);
+      lf_printf (file, "\n(next (insn_mnemonic_entry *) %p)", mnemonic->next);
       lf_indent (file, -1);
     }
   lf_printf (file, "%s", suffix);
@@ -1708,8 +1740,9 @@ dump_insn_mnemonic_entry (lf *file,
 
 static void
 dump_insn_mnemonic_entries (lf *file,
-			    char *prefix,
-			    insn_mnemonic_entry *mnemonic, char *suffix)
+			    const char *prefix,
+			    const insn_mnemonic_entry *mnemonic,
+			    const char *suffix)
 {
   lf_printf (file, "%s", prefix);
   while (mnemonic != NULL)
@@ -1721,9 +1754,12 @@ dump_insn_mnemonic_entries (lf *file,
 }
 
 void
-dump_insn_entry (lf *file, char *prefix, insn_entry * entry, char *suffix)
+dump_insn_entry (lf *file,
+		 const char *prefix,
+		 const insn_entry *entry,
+		 const char *suffix)
 {
-  lf_printf (file, "%s(insn_entry *) 0x%lx", prefix, (long) entry);
+  lf_printf (file, "%s(insn_entry *) %p", prefix, entry);
   if (entry != NULL)
     {
       int i;
@@ -1734,7 +1770,7 @@ dump_insn_entry (lf *file, char *prefix, insn_entry * entry, char *suffix)
       dump_insn_word_entries (file, "\n(words ", entry->words, ")");
       lf_printf (file, "\n(word");
       for (i = 0; i < entry->nr_models; i++)
-	lf_printf (file, " 0x%lx", (long) entry->word[i]);
+	lf_printf (file, " %p", entry->word[i]);
       lf_printf (file, ")");
       dump_filter (file, "\n(field_names ", entry->field_names, ")");
       lf_printf (file, "\n(format_name \"%s\")", entry->format_name);
@@ -1744,20 +1780,23 @@ dump_insn_entry (lf *file, char *prefix, insn_entry * entry, char *suffix)
       dump_insn_model_entries (file, "\n(models ", entry->models, ")");
       lf_printf (file, "\n(model");
       for (i = 0; i < entry->nr_models; i++)
-	lf_printf (file, " 0x%lx", (long) entry->model[i]);
+	lf_printf (file, " %p", entry->model[i]);
       lf_printf (file, ")");
       dump_filter (file, "\n(processors ", entry->processors, ")");
       dump_insn_mnemonic_entries (file, "\n(mnemonics ", entry->mnemonics,
 				  ")");
       dump_table_entry (file, "\n(code ", entry->code, ")");
-      lf_printf (file, "\n(next 0x%lx)", (long) entry->next);
+      lf_printf (file, "\n(next %p)", entry->next);
       lf_indent (file, -1);
     }
   lf_printf (file, "%s", suffix);
 }
 
 static void
-dump_insn_entries (lf *file, char *prefix, insn_entry * entry, char *suffix)
+dump_insn_entries (lf *file,
+		   const char *prefix,
+		   const insn_entry *entry,
+		   const char *suffix)
 {
   lf_printf (file, "%s", prefix);
   lf_indent (file, +1);
@@ -1771,11 +1810,13 @@ dump_insn_entries (lf *file, char *prefix, insn_entry * entry, char *suffix)
 }
 
 
-
 void
-dump_insn_table (lf *file, char *prefix, insn_table *isa, char *suffix)
+dump_insn_table (lf *file,
+		 const char *prefix,
+		 const insn_table *isa,
+		 const char *suffix)
 {
-  lf_printf (file, "%s(insn_table *) 0x%lx", prefix, (long) isa);
+  lf_printf (file, "%s(insn_table *) %p", prefix, isa);
   if (isa != NULL)
     {
       lf_indent (file, +1);
@@ -1803,7 +1844,7 @@ main (int argc, char **argv)
   insn_table *isa;
   lf *l;
 
-  INIT_OPTIONS (options);
+  INIT_OPTIONS ();
 
   if (argc == 3)
     filter_parse (&options.flags_filter, argv[2]);
