@@ -1,6 +1,6 @@
 /* gdb-if.c -- sim interface to GDB.
 
-Copyright (C) 2011-2022 Free Software Foundation, Inc.
+Copyright (C) 2011-2026 Free Software Foundation, Inc.
 Contributed by Red Hat, Inc.
 
 This file is part of the GNU simulators.
@@ -33,7 +33,7 @@ along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 #include "sim/callback.h"
 #include "sim/sim.h"
 #include "gdb/signals.h"
-#include "gdb/sim-rl78.h"
+#include "sim/sim-rl78.h"
 
 #include "cpu.h"
 #include "mem.h"
@@ -204,33 +204,33 @@ sim_create_inferior (SIM_DESC sd, struct bfd *abfd,
 
 /* Read memory.  */
 
-int
-sim_read (SIM_DESC sd, SIM_ADDR mem, void *buf, int length)
+uint64_t
+sim_read (SIM_DESC sd, uint64_t addr, void *buf, uint64_t length)
 {
   check_desc (sd);
 
-  if (mem >= MEM_SIZE)
+  if (addr >= MEM_SIZE)
     return 0;
-  else if (mem + length > MEM_SIZE)
-    length = MEM_SIZE - mem;
+  else if (addr + length > MEM_SIZE)
+    length = MEM_SIZE - addr;
 
-  mem_get_blk (mem, buf, length);
+  mem_get_blk (addr, buf, length);
   return length;
 }
 
 /* Write memory.  */
 
-int
-sim_write (SIM_DESC sd, SIM_ADDR mem, const void *buf, int length)
+uint64_t
+sim_write (SIM_DESC sd, uint64_t addr, const void *buf, uint64_t length)
 {
   check_desc (sd);
 
-  if (mem >= MEM_SIZE)
+  if (addr >= MEM_SIZE)
     return 0;
-  else if (mem + length > MEM_SIZE)
-    length = MEM_SIZE - mem;
+  else if (addr + length > MEM_SIZE)
+    length = MEM_SIZE - addr;
 
-  mem_put_blk (mem, buf, length);
+  mem_put_blk (addr, buf, length);
   return length;
 }
 
@@ -391,7 +391,7 @@ sim_store_register (SIM_DESC sd, int regno, const void *buf, int length)
 /* Print out message associated with "info target".  */
 
 void
-sim_info (SIM_DESC sd, int verbose)
+sim_info (SIM_DESC sd, bool verbose)
 {
   check_desc (sd);
 
